import shlex
from subprocess import Popen
from subprocess import PIPE

from .settings import PLATFORM

class Event:
    def __init__(self, event, icon_path=None, exec_cmd=None, exec_hotkey=None):

        self.icon_path = icon_path
        self.event = event
        self.exec_cmd = exec_cmd
        self.exec_hotkey = exec_hotkey

    def has_icon_path(self):
        return self.icon_path is not None

    def has_exec_cmd(self):
        return self.exec_cmd is not None

    def has_exec_hotkey(self):
        return self.exec_hotkey is not None

    def run_exec_cmd(self):
        if self.exec_cmd is not None:
            args = shlex.split(self.exec_cmd)
            p = Popen(args, stdout=PIPE, stderr=PIPE)
            return p

    def run_exec_hotkey(self):
        if self.exec_hotkey is not None:
            if PLATFORM == "Linux":
                args = self._run_exec_hotkey_linux()
            elif PLATFORM == "Darwin":
                args = self._run_exec_hotkey_darwin()
            else:
                raise OSError(f"I don't know how to execute keypresses on {PLATFORM}.")
            p = Popen(args,stdout=PIPE, stderr=PIPE)
            return p

    def _run_exec_hotkey_darwin(self):
        command = f"echo 'tell application \"System Events\" to keystroke \"{self.exec_hotkey}\"' | osascript"
        return shelx.split(command)

    def _run_exec_hotkey_linux(self):
        args = shlex.split(self.exec_hotkey)
        return ['xdotool', 'key'] + args









import os
import sys
import click
import socket
import pyqrcode
import logging

from pathlib import Path
from time import sleep
from flask import Flask, render_template, request, redirect, url_for, session
from waitress import serve

from .parser import Parser
from .auth import Auth

def shutdown_server():
    func = request.environ.get('werkzeug.server.shutdown')
    if func is None:
        raise RuntimeError('Not running with the Werkzeug Server')
    func()

def start_server(config_path=None, port_number=8800, verbosity=2, qrcode=False):
    # get the ip address
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))  # this sees if device is connected to internet
    ip = s.getsockname()[0]
    s.close()

    config_path = Path(config_path)


    events, auth_section = Parser(config_path).parse()
    auth = Auth(password=auth_section['password'])

    qr_path = Path(__file__).parent / 'static' / 'qr_code.svg'
    if auth.require_login:
        qr_url = f'http://{ip}:{port_number}/login?password={auth.password_b64}'
    else:
        qr_url = f'http://{ip}:{port_number}'
    qr = pyqrcode.create(qr_url)
    qr.svg(qr_path, module_color="#000000", background='#ffffff')
    if qrcode:
        print(qr.terminal())

    thqm = Flask(__name__.split('.')[0])
    thqm.secret_key = os.urandom(12)

    @thqm.route("/")
    def index():
        return render_template("index.html",
                               events=events.values(),
                               require_login=auth.require_login)

    @thqm.route("/login")
    def login():
        password = request.args.get("password", None)
        if password is not None:
            if auth.try_login(password):
                session['logged_in'] = True
        return redirect(url_for('index'))


    @thqm.route("/event")
    def do_event():
        # query is key and the default value is None
        key = request.args.get("key", "None")
        if verbosity >= 1:
            print(key, flush=True)

        # presses key as it receives via GET
        success = True
        if key == 'shutdown_rc_server':
            shutdown_server()

        try:
            events[key].run_exec_cmd()
            sleep(0.1)
            events[key].run_exec_hotkey()
            sleep(0.3)
        except Exception as e:
            print(e)
            success = False

        return {"press": success}

    serve(thqm, host="0.0.0.0", port=port_number, _quiet=verbosity <= 1)




import base64

class Auth:
    def __init__(self, password=None):
        self.password = password
        if password is not None:
            self.password_b64 = base64.b64encode(password.encode()).decode('utf8')
        else:
            self.password_b64=None
        self.require_login = password is not None

    def try_login(self, password):
        return password == self.password_b64




import os
import platform
from pathlib import Path

HOME = os.getenv('HOME', os.getenv('USERPROFILE'))
XDG_CONF_DIR = os.getenv('XDG_CONFIG_HOME', Path(HOME) / '.config')

CONF_DIR = Path(XDG_CONF_DIR) / 'thqm'
CONF_FILE = CONF_DIR / 'config'
# MODULE_DIR = Path(__file__).parent

CONF_FILE_DEFAULT = '''\
# This is the configuration file for thqm.
# Here is where you can configure the hotkeys or commands to provide on the
# thqm server.

# Here are some examples:

# If this block is present authentication will be required.
# [thqm-auth]
# password=thqm

# [Raise volume]
# exec_cmd=pactl set-sink-volume @DEFAULT_SINK@ +10%

# [Lower volume]
# exec_cmd=pactl set-sink-volume @DEFAULT_SINK@ -10%

# [Toggle mute]
# exec_cmd=pactl set-sink-mute @DEFAULT_SINK@ toggle

# [Media previous]
# exec_hotkey=XF86AudioPrev

# [Media next]
# exec_hotkey=XF86AudioNext

# [Media play]
# exec_hotkey=XF86AudioPlay

# [Media pause]
# exec_hotkey=XF86AudioPause

# [play/pause]
# exec_hotkey=space

# [scrub back]
# exec_hotkey=Left

# [scrub forward]
# exec_hotkey=Right
'''

PLATFORM = platform.uname()[0]




#!/usr/bin/env python
import argparse
import os
from pathlib import Path

from .server import start_server
from .settings import CONF_DIR, CONF_FILE, CONF_FILE_DEFAULT


def main():
    """Main logic.
    """
    CONF_DIR.mkdir(exist_ok=True)
    if not CONF_FILE.is_file():
        with open(CONF_FILE, 'w') as fp:
            fp.write(CONF_FILE_DEFAULT)

    parser = argparse.ArgumentParser(prog='thqm',
                                     description='Remote command and hotkey execution server.',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-c', '--config',
                        default=CONF_FILE,
                        help="Path to config file.")
    parser.add_argument("-p", "--port", type=int, default=8800,
                        help="Port number.")
    parser.add_argument('-q', '--qrcode', action='store_true',
                        default=False,
                        help='Show the qrcode.')
    parser.add_argument("-v", "--verbosity", action='count', default=0,
                        help=("Verbosity of the waitress server. "
                              "-v will print events. "
                              "-vv will print server messages."))
    args = parser.parse_args()

    if not Path(args.config).is_file():
        raise ValueError(f'File {args.config} does not exists.')

    start_server(args.config, port_number=args.port, qrcode=args.qrcode, verbosity=args.verbosity)

if __name__ == '__main__':
    main()



from configparser import ConfigParser
from pathlib import Path

from .event import Event

class Parser:
    def __init__(self, config_path):
        self.config_path = Path(config_path)

    def parse(self):
        cf = ConfigParser()
        cf.read(self.config_path)
        events = {}
        auth = {'password': None}
        for section in cf.sections():
            if section == 'thqm-auth':
                auth = cf._sections[section]
            else:
                events[section] = Event(section, **cf._sections[section])
        return events, auth




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['thqm']

package_data = \
{'': ['*'],
 'thqm': ['static/favicon.png',
          'static/favicon.png',
          'static/index.css',
          'static/index.css',
          'templates/*']}

install_requires = \
['flask>=1.1.1,<2.0.0', 'pyqrcode>=1.2.1,<2.0.0', 'waitress>=1.4.3,<2.0.0']

entry_points = \
{'console_scripts': ['thqm = thqm.__main__:main']}

setup_kwargs = {
    'name': 'thqm',
    'version': '1.0.0',
    'description': 'thqm, WSGI server providing simple remote hotkey and command execution on host.',
    'long_description': '# thqm\n> Control your desktop from your phone\n\n\n<p align="center">\n <img src="./images/logo.svg" width="200">\n</p>\n\n`thqm` (pronounced tahakoom from the arabic \xd8\xaa\xd8\xad\xd9\x83\xd9\x85  meaning control).\n\n`thqm` starts a `waitress` WSGI server on the host machine and allows the client to run pre-configured commands\n& hotkeys on the host machine. This allows for simple remote command & hotkey execution.\n\nA few nice use cases are:\n  * Media playback control\n  * Volume control\n  * Download/torrent control\n  * Control other web applications e.g. Jupyter notebook server\n\n# Installation\n```shell\npip install thqm\n```\n`thqm` is compatible with linux & MacOS (maybe windows ?, though probably not the hotkeys).\n\n# Dependencies\n\n  * `python3`\n  * `flask`\n  * `pyqrcode`\n  * `waitress`\n\nFor hotkey execution:\n  * linux: `xdotool`\n  * MacOS: `osascript`\n\n# Usage\n\nTo start the `thqm` server, on the host machine, simply run:\n```shell\nthqm\n```\nThe help:\n```\nusage: thqm [-h] [-c CONFIG] [-p PORT] [-q] [-v]\n\nRemote command and hotkey execution server.\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -c CONFIG, --config CONFIG\n                        Path to config file. (default:\n                        /home/lcoyle/.config/thqm/config)\n  -p PORT, --port PORT  Port number. (default: 8800)\n  -q, --qrcode          Show the qrcode. (default: False)\n  -v, --verbosity       Verbosity of the waitress server. -v will print\n                        events. -vv will print server messages. (default: 0)\n```\n\nThe `-q` flag will print a qr-code in the terminal, this qr-code will bypass the `thqm-auth` authentication, the same is true for the in browser qr-code, this makes it particularly easy to share access with others.\n<p align="center">\n <img src="./images/thqm_phone_portrait.png" width="200" />\n</p>\n\n# Configuration\n\n`thqm` reads a configuration file located at `$XDG_CONFIG_HOME/thqm/config` (or `$HOME/.config/thqm/config` if `$XDG_CONFIG_HOME` if not defined). This `ini` format file contains the pre-configured commands & hotkeys.\n\nEach `ini` block can have the following structure:\n```ini\n[event name]        # name of the event\nexec_cmd=...        # command to execute\nexec_hotkey=...     # hotkey to execute (on linux use xdotool key names, on macos use osascript key names)\nicon_path=...       # (optional) icon of the the event\n```\nif the event has both an `exec_cmd` and an `exec_hotkey` the command is run prior to running the hotkey.\n\nHere would be the contents for media playback and audio control:\n```ini\n# requires pactl\n[Raise volume]\nexec_cmd=pactl set-sink-volume @DEFAULT_SINK@ +10%\n\n[Lower volume]\nexec_cmd=pactl set-sink-volume @DEFAULT_SINK@ -10%\n\n[Toggle mute]\nexec_cmd=pactl set-sink-mute @DEFAULT_SINK@ toggle\n\n# using media keys\n[Media previous]\nexec_hotkey=XF86AudioPrev\n\n[Media next]\nexec_hotkey=XF86AudioNext\n\n[Media play]\nexec_hotkey=XF86AudioPlay\n\n[Media pause]\nexec_hotkey=XF86AudioPause\n\n# browser control\n[play/pause]\nexec_hotkey=space\n\n[scrub back]\nexec_hotkey=Left\n\n[scrub forward]\nexec_hotkey=Right\n```\n\nIf you want rudimentary authentication, add the `thqm-auth` block, with the `password` key, **don\'t use a real password, the encryption isn\'t secure**:\n```ini\n[thqm-auth]\npassword=super_secret_password\n```\n\n',
    'author': 'Loic Coyle',
    'author_email': 'loic.coyle@hotmail.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/loiccoyle/thqm',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
